/*
 * SendFromImpl.java
 *
 * Copyright 2014 the curewallet.org.
 * http://www.curewallet.org/
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.curewallet.lib.api.impl;

import java.math.BigDecimal;

import org.curewallet.lib.api.AbstractCoinAPI;
import org.curewallet.lib.api.CoinAPIException;
import org.curewallet.lib.api.SendFrom;
import org.curewallet.lib.api.response.SendFromResponseObject;
import org.curewallet.lib.api.result.SendFromResult;
import org.curewallet.lib.jsonrpc.JsonRPCClient;
import org.curewallet.lib.jsonrpc.JsonRPCv1RequestObject;

/**
 * 
 * 
 * @author Nezumi Kozo
 * @since 
 */
public class SendFromImpl extends AbstractCoinAPI implements SendFrom {

	protected static final String METHOD = "sendfrom";

	public SendFromImpl() {
	}

	/*
	 * (非 Javadoc)
	 * @see org.curewallet.lib.api.SendFrom#call(org.curewallet.lib.jsonrpc.JsonRPCClient, java.lang.String, java.lang.String, java.math.BigDecimal)
	 * @since 1.0
	 */
	public SendFromResult call(JsonRPCClient jsonRPCClient,
			String fromAccount, String toAddress, BigDecimal amount)
			throws CoinAPIException {
		return call(jsonRPCClient, fromAccount, toAddress, amount, null, null, null);
	}

	/*
	 * (非 Javadoc)
	 * @see org.curewallet.lib.api.SendFrom#call(org.curewallet.lib.jsonrpc.JsonRPCClient, java.lang.String, java.lang.String, java.math.BigDecimal, java.lang.String, java.lang.String)
	 * @since 1.0
	 */
	public SendFromResult call(JsonRPCClient jsonRPCClient,
			String fromAccount, String toAddress, BigDecimal amount,
			String comment, String commentTo) throws CoinAPIException {
		return call(jsonRPCClient, fromAccount, toAddress, amount, null, comment, commentTo);
	}

	/*
	 * (非 Javadoc)
	 * @see org.curewallet.lib.api.SendFrom#call(org.curewallet.lib.jsonrpc.JsonRPCClient, java.lang.String, java.lang.String, java.math.BigDecimal, java.lang.Long)
	 * @since 1.0
	 */
	public SendFromResult call(JsonRPCClient jsonRPCClient,
			String fromAccount, String toAddress, BigDecimal amount,
			Long minconf) throws CoinAPIException {
		return call(jsonRPCClient, fromAccount, toAddress, amount, minconf, null, null);
	}

	/*
	 * (非 Javadoc)
	 * @see org.curewallet.lib.api.SendFrom#call(org.curewallet.lib.jsonrpc.JsonRPCClient, java.lang.String, java.lang.String, java.math.BigDecimal, java.lang.Long, java.lang.String, java.lang.String)
	 * @since 1.0
	 */
	public SendFromResult call(JsonRPCClient jsonRPCClient,
			String fromAccount, String toAddress, BigDecimal amount,
			Long minconf, String comment, String commentTo)
			throws CoinAPIException {
		try {
			if (jsonRPCClient == null) {
				throw new NullPointerException("jsonRPCClient is null.");
			}
			if (fromAccount == null || fromAccount.length() == 0) {
				throw new NullPointerException("fromAccount is null.");
			}
			if (toAddress == null || fromAccount.length() == 0) {
				throw new NullPointerException("toAddress is null.");
			}
			if (amount == null) {
				throw new NullPointerException("amount is null.");
			}

			// request
			JsonRPCv1RequestObject requestObject = new JsonRPCv1RequestObject();
			requestObject.setMethod(METHOD);
			if (minconf == null && comment == null && commentTo == null) {
				requestObject.setParams(new Object[]{fromAccount, toAddress, amount});
			} else {
				if (minconf == null) {
					minconf = new Long(1);
				}
				if (comment == null) {
					comment = "";
				}
				if (commentTo == null) {
					commentTo = "";
				}
				requestObject.setParams(new Object[]{fromAccount, toAddress, amount, minconf, comment, commentTo});
			}

			// JSON-RPC call
			return jsonRPCClient.call(requestObject, SendFromResponseObject.class).getResultObject();

		} catch (Exception e) {
			// JsonRPCException
			throw new CoinAPIException("Error: " + e.getMessage(), e);
		}
	}

}
